-module(ai_player).
-behaviour(gen_server).

-export([start_link/1, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([create_ai_player/1, play_turn/1, new/1]).

-record(state, {
    name,
    game_pid,
    cards = [],
    difficulty = normal,  % easy | normal | hard
    strategy = undefined, % landlord | farmer
    last_play = []
}).

%% AI难度设置
-define(EASY_THINK_TIME, 1000).    % 1秒
-define(NORMAL_THINK_TIME, 500).   % 0.5秒
-define(HARD_THINK_TIME, 200).     % 0.2秒

%% API
start_link(Name) ->
    gen_server:start_link(?MODULE, [Name], []).

create_ai_player(Difficulty) ->
    Name = generate_ai_name(),
    {ok, Pid} = start_link(Name),
    gen_server:cast(Pid, {set_difficulty, Difficulty}),
    {ok, Pid}.

play_turn(AiPid) ->
    gen_server:cast(AiPid, play_turn).

%% 初始化AI玩家
new(Type) ->
    Name = generate_ai_name(),
    {ok, Pid} = start_link(Name),
    Difficulty = case Type of
        basic -> normal;
        advanced -> hard;
        _ -> normal
    end,
    gen_server:cast(Pid, {set_difficulty, Difficulty}),
    Pid.

%% Callbacks
init([Name]) ->
    {ok, #state{name = Name}}.

handle_call(get_name, _From, State) ->
    {reply, {ok, State#state.name}, State};

handle_call(_Request, _From, State) ->
    {reply, {error, unknown_call}, State}.

handle_cast({set_difficulty, Difficulty}, State) ->
    {noreply, State#state{difficulty = Difficulty}};

handle_cast({update_cards, Cards}, State) ->
    {noreply, State#state{cards = Cards}};

handle_cast({set_strategy, Strategy}, State) ->
    {noreply, State#state{strategy = Strategy}};

handle_cast(play_turn, State) ->
    timer:sleep(get_think_time(State#state.difficulty)),
    {Play, NewState} = calculate_best_play(State),
    case Play of
        pass ->
            game_server:pass(State#state.game_pid, self());
        Cards ->
            game_server:play_cards(State#state.game_pid, self(), Cards)
    end,
    {noreply, NewState};

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

% 终止回调
terminate(_Reason, _State) ->
    ok.

% 代码变更回调
code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部函数

% 生成AI玩家名称
generate_ai_name() ->
    Names = ["AlphaBot", "DeepPlayer", "SmartAI", "MasterBot", "ProBot"],
    RandomName = lists:nth(rand:uniform(length(Names)), Names),
    RandomNum = integer_to_list(rand:uniform(999)),
    RandomName ++ "_" ++ RandomNum.

% 获取思考时间
get_think_time(easy) -> ?EASY_THINK_TIME;
get_think_time(normal) -> ?NORMAL_THINK_TIME;
get_think_time(hard) -> ?HARD_THINK_TIME.

% 计算最佳出牌
calculate_best_play(State = #state{cards = Cards, strategy = Strategy, last_play = LastPlay}) ->
    case Strategy of
        landlord -> calculate_landlord_play(Cards, LastPlay, State);
        farmer -> calculate_farmer_play(Cards, LastPlay, State)
    end.

% 地主策略
calculate_landlord_play(Cards, LastPlay, State) ->
    case should_play_big(Cards, LastPlay, State) of
        true ->
            {get_biggest_play(Cards, LastPlay), State};
        false ->
            {get_optimal_play(Cards, LastPlay), State}
    end.

% 农民策略
calculate_farmer_play(Cards, LastPlay, State) ->
    case should_save_cards(Cards, LastPlay, State) of
        true ->
            {pass, State};
        false ->
            {get_optimal_play(Cards, LastPlay), State}
    end.

% 判断是否应该出大牌
should_play_big(Cards, LastPlay, _State) ->
    RemainingCount = length(Cards),
    case RemainingCount of
        1 -> true;
        2 -> true;
        _ -> 
            has_winning_chance(Cards, LastPlay)
    end.

% 判断是否应该保留手牌
should_save_cards(Cards, LastPlay, _State) ->
    case LastPlay of
        [] -> false;
        _ ->
            RemainingCount = length(Cards),
            OtherPlayerCards = estimate_other_players_cards(),
            RemainingCount < 5 andalso OtherPlayerCards > RemainingCount * 2
    end.

% 估算其他玩家的牌数
estimate_other_players_cards() ->
    15. % 这是一个简化的估算，实际实现需要根据游戏进程动态计算

% 检查是否有获胜机会
has_winning_chance(Cards, _LastPlay) ->
    % 简化版本的获胜概率计算
    length(Cards) =< 4.

% 获取最大的可出牌组合
get_biggest_play(Cards, LastPlay) ->
    % 实现查找最大牌型的逻辑
    % 这里需要调用 card_rules 模块来判断牌型
    case LastPlay of
        [] -> 
            find_biggest_combination(Cards);
        _ ->
            find_bigger_combination(Cards, LastPlay)
    end.

% 获取最优的出牌组合
get_optimal_play(Cards, LastPlay) ->
    % 实现最优出牌策略
    % 考虑因素：手牌数量、牌型组合、出牌节奏等
    case LastPlay of
        [] ->
            find_optimal_combination(Cards);
        _ ->
            find_minimum_bigger_combination(Cards, LastPlay)
    end.

% 查找最大牌型组合
find_biggest_combination(Cards) ->
    % 按照牌型优先级查找：火箭 > 炸弹 > 顺子 > 三带二 > 三带一 > 对子 > 单牌
    case find_rocket(Cards) of
        {ok, Rocket} -> Rocket;
        _ -> 
            case find_bomb(Cards) of
                {ok, Bomb} -> Bomb;
                _ -> find_best_normal_combination(Cards)
            end
    end.

% 查找比上家更大的牌型
find_bigger_combination(Cards, LastPlay) ->
    LastType = card_rules:get_card_type(LastPlay),
    case LastType of
        rocket -> []; % 火箭最大，无法压过
        bomb -> find_bigger_bomb(Cards, LastPlay);
        _ -> find_bigger_normal_combination(Cards, LastPlay, LastType)
    end.

% 查找最优组合
find_optimal_combination(Cards) ->
    % 简化实现：优先出单牌、对子、三带一等小牌型
    case find_single(Cards) of
        {ok, Single} -> Single;
        _ -> find_biggest_combination(Cards)
    end.

% 查找最小的能压过上家的牌
find_minimum_bigger_combination(Cards, LastPlay) ->
    % 简化实现：找到能压过上家的最小牌组合
    Bigger = find_bigger_combination(Cards, LastPlay),
    case Bigger of
        [] -> [];
        _ -> Bigger
    end.

% 查找火箭
find_rocket(Cards) ->
    % 检查是否同时有小王和大王
    HasSmallJoker = lists:any(fun(Card) -> Card =:= {"", "小王"} end, Cards),
    HasBigJoker = lists:any(fun(Card) -> Card =:= {"", "大王"} end, Cards),
    
    case HasSmallJoker andalso HasBigJoker of
        true -> {ok, [{"", "小王"}, {"", "大王"}]};
        false -> error
    end.

% 查找炸弹
find_bomb(Cards) ->
    % 按点数分组
    ValueGroups = lists:foldl(
        fun({_, Number}, Acc) ->
            Count = maps:get(Number, Acc, 0),
            Acc#{Number => Count + 1}
        end,
        #{},
        Cards
    ),
    
    % 查找数量为4的组
    BombValues = [Value || {Value, Count} <- maps:to_list(ValueGroups), Count >= 4],
    
    case BombValues of
        [] -> error;
        [Value|_] -> 
            % 找出这个点数的4张牌
            BombCards = lists:filter(fun({_, V}) -> V =:= Value end, Cards),
            {ok, lists:sublist(BombCards, 4)}
    end.

% 查找最佳普通组合
find_best_normal_combination(Cards) ->
    % 简化实现：按优先级尝试不同牌型
    % 实际应该实现更复杂的策略
    case find_straight(Cards) of
        {ok, Straight} -> Straight;
        _ ->
            case find_three_with_pair(Cards) of
                {ok, ThreeWithPair} -> ThreeWithPair;
                _ ->
                    case find_three(Cards) of
                        {ok, Three} -> Three;
                        _ ->
                            case find_pair(Cards) of
                                {ok, Pair} -> Pair;
                                _ ->
                                    case find_single(Cards) of
                                        {ok, Single} -> Single;
                                        _ -> []
                                    end
                            end
                    end
            end
    end.

% 查找比上家更大的炸弹
find_bigger_bomb(Cards, LastPlay) ->
    % 获取上家炸弹的点数
    [{_, LastValue}|_] = LastPlay,
    
    % 查找更大的炸弹
    {ok, MyBomb} = find_bomb(Cards),
    [{_, MyValue}|_] = MyBomb,
    
    % 比较点数
    case card_value(MyValue) > card_value(LastValue) of
        true -> MyBomb;
        false -> []
    end.

% 查找比上家更大的普通组合
find_bigger_normal_combination(Cards, LastPlay, Type) ->
    % 简化实现：根据牌型查找更大的组合
    % 实际应该针对每种牌型实现专门的查找逻辑
    case Type of
        single -> find_bigger_single(Cards, LastPlay);
        pair -> find_bigger_pair(Cards, LastPlay);
        triple -> find_bigger_triple(Cards, LastPlay);
        straight -> find_bigger_straight(Cards, LastPlay);
        _ -> []
    end.

% 查找更大的对子
find_bigger_pair(Cards, LastPlay) ->
    [{_, LastValue}|_] = LastPlay,
    LastValueNum = card_value(LastValue),
    % 按点数分组
    ValueGroups = lists:foldl(
        fun({_, Number}, Acc) ->
            Count = maps:get(Number, Acc, 0),
            Acc#{Number => Count + 1}
        end,
        #{},
        Cards
    ),
    % 查找大于LastValue且数量至少为2的组
    BiggerPairValues = [Value || {Value, Count} <- maps:to_list(ValueGroups),
                                Count >= 2, card_value(Value) > LastValueNum],
    case BiggerPairValues of
        [] -> [];
        _ -> 
            % 找出最小的大于LastValue的对子
            SortedValues = lists:sort(
                fun(V1, V2) -> card_value(V1) < card_value(V2) end,
                BiggerPairValues
            ),
            SmallestValue = hd(SortedValues),
            PairCards = lists:filter(fun({_, V}) -> V =:= SmallestValue end, Cards),
            lists:sublist(PairCards, 2)
    end.

% 查找更大的三张
find_bigger_triple(Cards, LastPlay) ->
    [{_, LastValue}|_] = LastPlay,
    LastValueNum = card_value(LastValue),
    % 按点数分组
    ValueGroups = lists:foldl(
        fun({_, Number}, Acc) ->
            Count = maps:get(Number, Acc, 0),
            Acc#{Number => Count + 1}
        end,
        #{},
        Cards
    ),
    % 查找大于LastValue且数量至少为3的组
    BiggerTripleValues = [Value || {Value, Count} <- maps:to_list(ValueGroups),
                                 Count >= 3, card_value(Value) > LastValueNum],
    case BiggerTripleValues of
        [] -> [];
        _ -> 
            % 找出最小的大于LastValue的三张
            SortedValues = lists:sort(
                fun(V1, V2) -> card_value(V1) < card_value(V2) end,
                BiggerTripleValues
            ),
            SmallestValue = hd(SortedValues),
            TripleCards = lists:filter(fun({_, V}) -> V =:= SmallestValue end, Cards),
            lists:sublist(TripleCards, 3)
    end.

% 查找更大的顺子
find_bigger_straight(Cards, LastPlay) ->
    % 简化实现，实际应该有更复杂的逻辑
    [].

% 查找单牌
find_single(Cards) ->
    case Cards of
        [] -> error;
        [Card|_] -> {ok, [Card]}
    end.

% 查找对子
find_pair(Cards) ->
    % 按点数分组
    ValueGroups = lists:foldl(
        fun({_, Number}, Acc) ->
            Count = maps:get(Number, Acc, 0),
            Acc#{Number => Count + 1}
        end,
        #{},
        Cards
    ),
    
    % 查找数量至少为2的组
    PairValues = [Value || {Value, Count} <- maps:to_list(ValueGroups), Count >= 2],
    
    case PairValues of
        [] -> error;
        [Value|_] -> 
            % 找出这个点数的2张牌
            PairCards = lists:filter(fun({_, V}) -> V =:= Value end, Cards),
            {ok, lists:sublist(PairCards, 2)}
    end.

% 查找三张
find_three(Cards) ->
    % 按点数分组
    ValueGroups = lists:foldl(
        fun({_, Number}, Acc) ->
            Count = maps:get(Number, Acc, 0),
            Acc#{Number => Count + 1}
        end,
        #{},
        Cards
    ),
    
    % 查找数量至少为3的组
    ThreeValues = [Value || {Value, Count} <- maps:to_list(ValueGroups), Count >= 3],
    
    case ThreeValues of
        [] -> error;
        [Value|_] -> 
            % 找出这个点数的3张牌
            ThreeCards = lists:filter(fun({_, V}) -> V =:= Value end, Cards),
            {ok, lists:sublist(ThreeCards, 3)}
    end.

% 查找三带二
find_three_with_pair(Cards) ->
    % 简化实现
    case find_three(Cards) of
        {ok, Three} ->
            RemainingCards = Cards -- Three,
            case find_pair(RemainingCards) of
                {ok, Pair} -> {ok, Three ++ Pair};
                _ -> error
            end;
        _ -> error
    end.

% 查找顺子
find_straight(Cards) ->
    % 简化实现，实际应该有更复杂的逻辑
    error.

% 获取牌的点数值
card_value("大王") -> 17;
card_value("小王") -> 16;
card_value("2") -> 15;
card_value("A") -> 14;
card_value("K") -> 13;
card_value("Q") -> 12;
card_value("J") -> 11;
card_value(Number) when is_list(Number) ->
    try
        list_to_integer(Number)
    catch
        error:_ -> 0
    end.

% 查找更大的单牌
find_bigger_single(Cards, LastPlay) ->
    [{_, LastValue}|_] = LastPlay,
    LastValueNum = card_value(LastValue),
    % 找出所有大于LastValue的单牌
    BiggerCards = lists:filter(
        fun({_, Value}) -> card_value(Value) > LastValueNum end,
        Cards
    ),
    case BiggerCards of
        [] -> [];
        _ -> 
            % 找出最小的大于LastValue的牌
            SortedBiggerCards = lists:sort(
                fun({_, V1}, {_, V2}) -> card_value(V1) < card_value(V2) end,
                BiggerCards
            ),
            [hd(SortedBiggerCards)]
    end.